@@ -38,6 +38,7 @@ import org.picocontainer.containers.ImmutablePicoContainer;
 import org.picocontainer.injectors.AbstractInjector;
 import org.picocontainer.injectors.AdaptingInjection;
 import org.picocontainer.injectors.FactoryInjector;
+import org.picocontainer.injectors.InjectInto;
 import org.picocontainer.lifecycle.DefaultLifecycleState;
 import org.picocontainer.lifecycle.LifecycleState;
 import org.picocontainer.lifecycle.StartableLifecycleStrategy;
@@ -266,11 +267,15 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
         return Collections.unmodifiableList(getModifiableComponentAdapterList());
     }
 
+
     /** {@inheritDoc} **/
     public final ComponentAdapter<?> getComponentAdapter(final Object componentKey) {
         ComponentAdapter<?> adapter = getComponentKeyToAdapterCache().get(componentKey);
         if (adapter == null && parent != null) {
             adapter = getParent().getComponentAdapter(componentKey);
+            if (adapter != null) {
+                adapter = new KnowsContainerAdapter(adapter, getParent());
+            }
         }
         if (adapter == null) {
             Object inst = componentMonitor.noComponentFound(this, componentKey);
@@ -304,6 +309,55 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
         }
     }
 
+    public static class KnowsContainerAdapter<T> implements ComponentAdapter<T> {
+        private final ComponentAdapter<T> ca;
+        private final PicoContainer ctr;
+
+        public KnowsContainerAdapter(ComponentAdapter<T> ca, PicoContainer ctr) {
+            this.ca = ca;
+            this.ctr = ctr;
+        }
+
+        public T getComponentInstance(Type into) throws PicoCompositionException {
+            return getComponentInstance(ctr, into);
+        }
+
+        public Object getComponentKey() {
+            return ca.getComponentKey();
+        }
+
+        public Class getComponentImplementation() {
+            return ca.getComponentImplementation();
+        }
+
+        public T getComponentInstance(PicoContainer container) throws PicoCompositionException {
+            return ca.getComponentInstance(container);
+        }
+
+        public T getComponentInstance(PicoContainer container, Type into) throws PicoCompositionException {
+            return ca.getComponentInstance(container, into);
+        }
+
+        public void verify(PicoContainer container) throws PicoCompositionException {
+            ca.verify(container);
+        }
+
+        public void accept(PicoVisitor visitor) {
+            ca.accept(visitor);
+        }
+
+        public ComponentAdapter getDelegate() {
+            return ca.getDelegate();
+        }
+
+        public <U extends ComponentAdapter> U findAdapterOfType(Class<U> adapterType) {
+            return ca.findAdapterOfType(adapterType);
+        }
+
+        public String getDescriptor() {
+            return null;
+        }
+    }
 
     /** {@inheritDoc} **/
     public <T> ComponentAdapter<T> getComponentAdapter(final Class<T> componentType, final NameBinding componentNameBinding) {
@@ -349,6 +403,7 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
 
     /** {@inheritDoc} **/
     public <T> ComponentAdapter<T> getComponentAdapter(final Class<T> componentType, final Class<? extends Annotation> binding) {
+        // 1
         return getComponentAdapter(componentType, null, binding);
     }
 
@@ -475,7 +530,7 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
         
         if (componentImplementationOrInstance instanceof Class) {
             Properties tmpProperties = (Properties) properties.clone();
-            ComponentAdapter<?> componentAdapter = componentFactory.createComponentAdapter(componentMonitor,
+            ComponentAdapter<?> adapter = componentFactory.createComponentAdapter(componentMonitor,
                                                                                                lifecycleStrategy,
                                                                                                tmpProperties,
                                                                                                componentKey,
@@ -483,11 +538,19 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
                                                                                                parameters);
             AbstractBehaviorFactory.removePropertiesIfPresent(tmpProperties, Characteristics.USE_NAMES);
             throwIfPropertiesLeft(tmpProperties);
-            return addAdapterInternal(componentAdapter);
+            if (lifecycleState.isStarted()) {
+                addAdapterIfStartable(adapter);
+                potentiallyStartAdapter(adapter);
+            }
+            return addAdapterInternal(adapter);
         } else {
-            ComponentAdapter<?> componentAdapter =
+            ComponentAdapter<?> adapter =
                 new InstanceAdapter<Object>(componentKey, componentImplementationOrInstance, lifecycleStrategy, componentMonitor);
-            return addAdapter(componentAdapter, properties);
+            if (lifecycleState.isStarted()) {
+                addAdapterIfStartable(adapter);
+                potentiallyStartAdapter(adapter);
+            }
+            return addAdapter(adapter, properties);
         }
     }
 
@@ -563,17 +626,35 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
     }
 
     public Object getComponent(final Object componentKeyOrType, final Class<? extends Annotation> annotation) {
-        Object retVal;
+        ComponentAdapter<?> componentAdapter;
+        Object component;
         if (annotation != null) {
-            final ComponentAdapter<?> componentAdapter = getComponentAdapter((Class<?>)componentKeyOrType, annotation);
-            return componentAdapter == null ? null : getInstance(componentAdapter, null);
+            componentAdapter = getComponentAdapter((Class<?>)componentKeyOrType, annotation);
+            component = componentAdapter == null ? null : getInstance(componentAdapter, null);
         } else if (componentKeyOrType instanceof Class) {
-            final ComponentAdapter<?> componentAdapter = getComponentAdapter((Class<?>)componentKeyOrType, (NameBinding) null);
-            return componentAdapter == null ? null : getInstance(componentAdapter, (Class<?>)componentKeyOrType);
+            componentAdapter = getComponentAdapter((Class<?>)componentKeyOrType, (NameBinding) null);
+            component = componentAdapter == null ? null : getInstance(componentAdapter, (Class<?>)componentKeyOrType);
         } else {
-            ComponentAdapter<?> componentAdapter = getComponentAdapter(componentKeyOrType);
-            return componentAdapter == null ? null : getInstance(componentAdapter, null);
+            componentAdapter = getComponentAdapter(componentKeyOrType);
+            component = componentAdapter == null ? null : getInstance(componentAdapter, null);
         }
+        return decorateComponent(component, componentAdapter);
+    }
+
+    /**
+     * This is invoked when getComponent(..) is called.  It allows extendees to decorate a
+     * component before it is returned to the caller.
+     * @param component the component that will be returned for getComponent(..)
+     * @param componentAdapter the component adapter that made that component
+     * @return the component (the same as that passed in by default)
+     */
+    protected Object decorateComponent(Object component, ComponentAdapter<?> componentAdapter) {
+        if (componentAdapter instanceof ComponentLifecycle<?>
+                && lifecycleStrategy.isLazy(componentAdapter) // is Lazy
+                && !((ComponentLifecycle<?>) componentAdapter).isStarted()) {
+            ((ComponentLifecycle<?>)componentAdapter).start(this);
+        }
+        return component;
     }
 
     public <T> T getComponent(final Class<T> componentType) {
@@ -582,7 +663,7 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
     }
 
     public <T> T getComponent(final Class<T> componentType, final Class<? extends Annotation> binding) {
-         Object o = getComponent((Object)componentType, binding);
+        Object o = getComponent((Object)componentType, binding);
         return componentType.cast(o);
     }
 
@@ -592,7 +673,7 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
         // we need to check this to ensure up-down dependencies cannot be followed
         final boolean isLocal = getModifiableComponentAdapterList().contains(componentAdapter);
 
-        if (isLocal) {
+        if (isLocal || componentAdapter instanceof LateInstance) {
             Object instance;
             try {
                 if (componentAdapter instanceof FactoryInjector) {
@@ -603,7 +684,7 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
                             intoThreadLocal = new IntoThreadLocal();
                         }
                     }
-                    intoThreadLocal.set(componentAdapter.getComponentImplementation());
+                    intoThreadLocal.set(new InjectInto(componentAdapter.getComponentImplementation(), componentAdapter.getComponentKey()));
                     instance = componentAdapter.getComponentInstance(this, ComponentAdapter.NOTHING.class);
                 }
             } catch (AbstractInjector.CyclicDependencyException e) {
@@ -898,26 +979,41 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
     private void startAdapters() {
         Collection<ComponentAdapter<?>> adapters = getComponentAdapters();
         for (ComponentAdapter<?> adapter : adapters) {
-            if (adapter instanceof ComponentLifecycle) {
-                ComponentLifecycle<?> componentLifecycle = (ComponentLifecycle<?>)adapter;
-                if (componentLifecycle.componentHasLifecycle()) {
-                    // create an instance, it will be added to the ordered CA list
-                    adapter.getComponentInstance(DefaultPicoContainer.this, ComponentAdapter.NOTHING.class);
-                    addOrderedComponentAdapter(adapter);
-                }
-            }
+            addAdapterIfStartable(adapter);
         }
         adapters = getOrderedComponentAdapters();
         // clone the adapters
         List<ComponentAdapter<?>> adaptersClone = new ArrayList<ComponentAdapter<?>>(adapters);
         for (final ComponentAdapter<?> adapter : adaptersClone) {
-            if (adapter instanceof ComponentLifecycle) {
-                ComponentLifecycle<?> componentLifecycle = (ComponentLifecycle<?>)adapter;
-                componentLifecycle.start(DefaultPicoContainer.this);
+            potentiallyStartAdapter(adapter);
+        }
+    }
+
+    protected void potentiallyStartAdapter(ComponentAdapter<?> adapter) {
+        if (adapter instanceof ComponentLifecycle) {
+            if (!lifecycleStrategy.isLazy(adapter)) {
+                ((ComponentLifecycle<?>)adapter).start(this);
             }
         }
     }
 
+    private void addAdapterIfStartable(ComponentAdapter<?> adapter) {
+        if (adapter instanceof ComponentLifecycle) {
+            ComponentLifecycle<?> componentLifecycle = (ComponentLifecycle<?>)adapter;
+            if (componentLifecycle.componentHasLifecycle()) {
+                // create an instance, it will be added to the ordered CA list
+                instantiateComponentAsIsStartable(adapter);
+                addOrderedComponentAdapter(adapter);
+            }
+        }
+    }
+
+    protected void instantiateComponentAsIsStartable(ComponentAdapter<?> adapter) {
+        if (!lifecycleStrategy.isLazy(adapter)) {
+            adapter.getComponentInstance(DefaultPicoContainer.this, ComponentAdapter.NOTHING.class);
+        }
+    }
+
     /**
      * {@inheritDoc}
      * Loops over started component adapters (in inverse order) and invokes
