@@ -26,6 +26,7 @@ import static org.picocontainer.injectors.Injectors.adaptiveDI;
 import org.picocontainer.lifecycle.NullLifecycleStrategy;
 import org.picocontainer.lifecycle.ReflectionLifecycleStrategy;
 import org.picocontainer.lifecycle.StartableLifecycleStrategy;
+import org.picocontainer.lifecycle.JavaEE5LifecycleStrategy;
 import org.picocontainer.monitors.ConsoleComponentMonitor;
 import org.picocontainer.monitors.NullComponentMonitor;
 
@@ -51,6 +52,12 @@ public class PicoBuilder {
     private ComponentMonitor componentMonitor;
     private List<Object> containerComps = new ArrayList<Object>();
     private boolean addChildToParent;
+    private LifecycleStrategy lifecycleStrategy;
+    private final Stack<Object> componentFactories = new Stack<Object>();
+    private InjectionFactory injectionType;
+    private Class<? extends ComponentMonitor> componentMonitorClass = NullComponentMonitor.class;
+    private Class<? extends LifecycleStrategy> lifecycleStrategyClass = NullLifecycleStrategy.class;
+
 
     public PicoBuilder(PicoContainer parentContainer, InjectionFactory injectionType) {
         this.injectionType = injectionType;
@@ -73,23 +80,37 @@ public class PicoBuilder {
         this(new EmptyPicoContainer(), adaptiveDI());
     }
 
-    private final Stack<Object> componentFactories = new Stack<Object>();
-
-    private InjectionFactory injectionType;
-
-    private Class<? extends ComponentMonitor> componentMonitorClass = NullComponentMonitor.class;
-    private Class<? extends LifecycleStrategy> lifecycleStrategyClass = NullLifecycleStrategy.class;
-
     public PicoBuilder withLifecycle() {
         lifecycleStrategyClass = StartableLifecycleStrategy.class;
+        lifecycleStrategy = null;
         return this;
     }
 
     public PicoBuilder withReflectionLifecycle() {
         lifecycleStrategyClass = ReflectionLifecycleStrategy.class;
+        lifecycleStrategy = null;
         return this;
     }
 
+    public PicoBuilder withLifecycle(Class<? extends LifecycleStrategy> lifecycleStrategyClass) {
+        this.lifecycleStrategyClass = lifecycleStrategyClass;
+        lifecycleStrategy = null;
+        return this;
+    }
+
+    public PicoBuilder withJavaEE5Lifecycle() {
+        this.lifecycleStrategyClass = JavaEE5LifecycleStrategy.class;
+        lifecycleStrategy = null;
+        return this;
+    }
+
+    public PicoBuilder withLifecycle(LifecycleStrategy lifecycleStrategy) {
+        this.lifecycleStrategy = lifecycleStrategy;
+        lifecycleStrategyClass = null;
+        return this;
+    }
+
+
     public PicoBuilder withConsoleMonitor() {
         componentMonitorClass =  ConsoleComponentMonitor.class;
         return this;
@@ -124,7 +145,12 @@ public class PicoBuilder {
 
         buildComponentMonitor(tempContainer);
 
-        tempContainer.addComponent(LifecycleStrategy.class, lifecycleStrategyClass);
+        if (lifecycleStrategy == null) {
+            tempContainer.addComponent(LifecycleStrategy.class, lifecycleStrategyClass);
+        } else {
+            tempContainer.addComponent(LifecycleStrategy.class, lifecycleStrategy);
+
+        }
         tempContainer.addComponent("mpc", mpcClass);
 
         MutablePicoContainer newContainer = (MutablePicoContainer) tempContainer.getComponent("mpc");
