@@ -9,12 +9,7 @@
  *****************************************************************************/
 package org.picocontainer.adapters;
 
-import org.picocontainer.Behavior;
-import org.picocontainer.PicoContainer;
-import org.picocontainer.LifecycleStrategy;
-import org.picocontainer.ComponentMonitor;
-import org.picocontainer.PicoCompositionException;
-import org.picocontainer.ComponentLifecycle;
+import org.picocontainer.*;
 import org.picocontainer.adapters.AbstractAdapter;
 import org.picocontainer.lifecycle.NullLifecycleStrategy;
 import org.picocontainer.monitors.NullComponentMonitor;
@@ -127,5 +122,7 @@ public final class InstanceAdapter<T> extends AbstractAdapter<T> implements Comp
         return lifecycleStrategy.hasLifecycle(type);
     }
 
-
+    public boolean isLazy(ComponentAdapter<?> adapter) {
+        return lifecycleStrategy.isLazy(adapter);
+    }
 }
