@@ -62,6 +62,6 @@ public class AnnotatedMethodInjection extends AbstractInjectionFactory {
     public <T> ComponentAdapter<T> createComponentAdapter(ComponentMonitor componentMonitor, LifecycleStrategy lifecycleStrategy, Properties componentProperties,
                                                    Object componentKey, Class<T> componentImplementation, Parameter... parameters)
             throws PicoCompositionException {
-        return new AnnotatedMethodInjector(componentKey, componentImplementation, parameters, componentMonitor, lifecycleStrategy, injectionAnnotation, useNames);
+        return componentMonitor.newInjector(new AnnotatedMethodInjector(componentKey, componentImplementation, parameters, componentMonitor, lifecycleStrategy, injectionAnnotation, useNames));
     }
 }
\ No newline at end of file
