@@ -49,9 +49,9 @@ public class CompositeInjection extends AbstractInjectionFactory {
                     lifecycleStrategy, componentProperties, componentKey, componentImplementation, parameters);
         }
 
-        boolean useNames = AbstractBehaviorFactory.arePropertiesPresent(componentProperties, Characteristics.USE_NAMES);
-        return new CompositeInjector(componentKey, componentImplementation, parameters,
+        boolean useNames = AbstractBehaviorFactory.arePropertiesPresent(componentProperties, Characteristics.USE_NAMES, true);
+        return componentMonitor.newInjector(new CompositeInjector(componentKey, componentImplementation, parameters,
                 componentMonitor, lifecycleStrategy,
-                useNames, injectors);
+                useNames, injectors));
     }
 }
\ No newline at end of file
