@@ -31,10 +31,12 @@ public class CompositeInjector<T> extends AbstractInjector<T> {
     }
 
 
+    @Override
     public T getComponentInstance(PicoContainer container) throws PicoCompositionException {
         return getComponentInstance(container, NOTHING.class);
     }
 
+    @Override
     public T getComponentInstance(PicoContainer container, Type into) throws PicoCompositionException {
         T instance = null;
         for (int i = 0; i < injectors.length; i++) {
@@ -49,15 +51,26 @@ public class CompositeInjector<T> extends AbstractInjector<T> {
     }
 
 
-    public void decorateComponentInstance(PicoContainer container, Type into, T instance) {
+    /**
+     * @return the object returned is the result of the last of the injectors delegated to
+     */
+    @Override
+    public Object decorateComponentInstance(PicoContainer container, Type into, T instance) {
+        Object result = null;
+        for (int i = 0; i < injectors.length; i++) {
+            result = injectors[i].decorateComponentInstance(container, into, instance);
+        }        
+        return result;
     }
 
+    @Override
     public void verify(PicoContainer container) throws PicoCompositionException {
         for (int i = 0; i < injectors.length; i++) {
             injectors[i].verify(container);
         }
     }
 
+    @Override
     public final void accept(PicoVisitor visitor) {
         super.accept(visitor);
         for (int i = 0; i < injectors.length; i++) {
@@ -65,6 +78,7 @@ public class CompositeInjector<T> extends AbstractInjector<T> {
         }
     }
 
+    @Override
     public String getDescriptor() {
         return "CompositeInjector";
     }
