@@ -15,6 +15,8 @@ import org.picocontainer.LifecycleStrategy;
 import org.picocontainer.Parameter;
 import org.picocontainer.PicoCompositionException;
 import org.picocontainer.PicoContainer;
+import org.picocontainer.ComponentAdapter;
+import org.picocontainer.NameBinding;
 import org.picocontainer.lifecycle.NullLifecycleStrategy;
 import org.picocontainer.monitors.NullComponentMonitor;
 
@@ -33,6 +35,8 @@ import java.util.Comparator;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
+import java.util.Map;
+import java.util.HashMap;
 
 /**
  * Injection will happen through a constructor for the component.
@@ -50,7 +54,7 @@ public class ConstructorInjector<T> extends SingleMemberInjector<T> {
 	private transient List<Constructor<T>> sortedMatchingConstructors;
     private transient ThreadLocalCyclicDependencyGuard<T> instantiationGuard;
     private boolean rememberChosenConstructor = true;
-    private transient Constructor<T> chosenConstructor;
+    private transient CtorAndAdapters<T> chosenConstructor;
 
     /**
      * Constructor injector that uses no monitor and no lifecycle adapter.  This is a more
@@ -101,30 +105,59 @@ public class ConstructorInjector<T> extends SingleMemberInjector<T> {
         this.rememberChosenConstructor = rememberChosenCtor;
     }
 
-    protected Constructor<T> getGreediestSatisfiableConstructor(PicoContainer container) throws PicoCompositionException {
+    private CtorAndAdapters<T> getGreediestSatisfiableConstructor(PicoContainer guardedContainer, @SuppressWarnings("unused") Class<T> componentImplementation) {
+        CtorAndAdapters<T> ctor = null;
+        try {
+            if (chosenConstructor == null) {
+                ctor = getGreediestSatisfiableConstructor(guardedContainer);
+            }
+            if (rememberChosenConstructor) {
+                if (chosenConstructor == null) {
+                    chosenConstructor = ctor;
+                } else {
+                    ctor = chosenConstructor;
+                }
+            }
+        } catch (AmbiguousComponentResolutionException e) {
+            e.setComponent(getComponentImplementation());
+            throw e;
+        }
+        return ctor;
+    }
+
+    @SuppressWarnings("synthetic-access")
+    protected CtorAndAdapters<T> getGreediestSatisfiableConstructor(PicoContainer container) throws PicoCompositionException {
         final Set<Constructor> conflicts = new HashSet<Constructor>();
         final Set<List<Type>> unsatisfiableDependencyTypes = new HashSet<List<Type>>();
+        final Map<ResolverKey, Parameter.Resolver> resolvers = new HashMap<ResolverKey, Parameter.Resolver>();
         if (sortedMatchingConstructors == null) {
             sortedMatchingConstructors = getSortedMatchingConstructors();
         }
         Constructor<T> greediestConstructor = null;
+        Parameter[] greediestConstructorsParameters = null;
+        ComponentAdapter[] greediestConstructorsParametersComponentAdapters = null;
         int lastSatisfiableConstructorSize = -1;
         Type unsatisfiedDependencyType = null;
         for (final Constructor<T> sortedMatchingConstructor : sortedMatchingConstructors) {
             boolean failedDependency = false;
             Type[] parameterTypes = sortedMatchingConstructor.getGenericParameterTypes();
-            fixParameterType(sortedMatchingConstructor, parameterTypes);
+            fixGenericParameterTypes(sortedMatchingConstructor, parameterTypes);
             Annotation[] bindings = getBindings(sortedMatchingConstructor.getParameterAnnotations());
-            Parameter[] currentParameters = parameters != null ? parameters : createDefaultParameters(parameterTypes);
-
+            final Parameter[] currentParameters = parameters != null ? parameters : createDefaultParameters(parameterTypes);
+            final ComponentAdapter<?>[] currentAdapters = new ComponentAdapter<?>[currentParameters.length];
             // remember: all constructors with less arguments than the given parameters are filtered out already
             for (int j = 0; j < currentParameters.length; j++) {
                 // check whether this constructor is satisfiable
-                Type boxed = box(parameterTypes[j]);
-                boolean un = useNames();
-                if (currentParameters[j].isResolvable(container, this, boxed,
-                        new ParameterNameBinding(getParanamer(), getComponentImplementation(), sortedMatchingConstructor, j),
-                        un, bindings[j])) {
+                Type expectedType = box(parameterTypes[j]);
+                NameBinding expectedNameBinding = new ParameterNameBinding(getParanamer(), sortedMatchingConstructor, j);
+                ResolverKey resolverKey = new ResolverKey(expectedType, useNames() ? expectedNameBinding.getName() : null, useNames(), bindings[j], currentParameters[j]);
+                Parameter.Resolver resolver = resolvers.get(resolverKey);
+                if (resolver == null) {
+                    resolver = currentParameters[j].resolve(container, this, null, expectedType, expectedNameBinding, useNames(), bindings[j]);
+                    resolvers.put(resolverKey, resolver);
+                }
+                if (resolver.isResolved()) {
+                    currentAdapters[j] = resolver.getComponentAdapter();
                     continue;
                 }
                 unsatisfiableDependencyTypes.add(Arrays.asList(parameterTypes));
@@ -136,17 +169,18 @@ public class ConstructorInjector<T> extends SingleMemberInjector<T> {
             if (greediestConstructor != null && parameterTypes.length != lastSatisfiableConstructorSize) {
                 if (conflicts.isEmpty()) {
                     // we found our match [aka. greedy and satisfied]
-                    return greediestConstructor;
-                } else {
-                    // fits although not greedy
-                    conflicts.add(sortedMatchingConstructor);
-                }
+                    return new CtorAndAdapters<T>(greediestConstructor, greediestConstructorsParameters, greediestConstructorsParametersComponentAdapters);
+                } 
+                // fits although not greedy
+                conflicts.add(sortedMatchingConstructor);
             } else if (!failedDependency && lastSatisfiableConstructorSize == parameterTypes.length) {
                 // satisfied and same size as previous one?
                 conflicts.add(sortedMatchingConstructor);
                 conflicts.add(greediestConstructor);
             } else if (!failedDependency) {
                 greediestConstructor = sortedMatchingConstructor;
+                greediestConstructorsParameters = currentParameters;
+                greediestConstructorsParametersComponentAdapters = currentAdapters;
                 lastSatisfiableConstructorSize = parameterTypes.length;
             }
         }
@@ -162,51 +196,127 @@ public class ConstructorInjector<T> extends SingleMemberInjector<T> {
             }
             throw new PicoCompositionException("Either the specified parameters do not match any of the following constructors: " + nonMatching.toString() + "; OR the constructors were not accessible for '" + getComponentImplementation().getName() + "'");
         }
-        return greediestConstructor;
+        return new CtorAndAdapters<T>(greediestConstructor, greediestConstructorsParameters, greediestConstructorsParametersComponentAdapters);
     }
 
-    private void fixParameterType(Constructor<T> sortedMatchingConstructor, Type[] parameterTypes) {
+    private static final class ResolverKey {
+        private final Type expectedType;
+        private final String pName;
+        private final boolean useNames;
+        private final Annotation binding;
+        private final Parameter currentParameter;
+
+        private ResolverKey(Type expectedType, String pName, boolean useNames, Annotation binding, Parameter currentParameter) {
+            this.expectedType = expectedType;
+            this.pName = pName;
+            this.useNames = useNames;
+            this.binding = binding;
+            this.currentParameter = currentParameter;
+        }
+
+        // Generated by IDEA
+        @Override
+        public boolean equals(Object o) {
+            if (this == o) return true;
+            if (o == null || getClass() != o.getClass()) return false;
+
+            ResolverKey that = (ResolverKey) o;
+
+            if (useNames != that.useNames) return false;
+            if (binding != null ? !binding.equals(that.binding) : that.binding != null) return false;
+            if (!currentParameter.equals(that.currentParameter)) return false;
+            if (!expectedType.equals(that.expectedType)) return false;
+            if (pName != null ? !pName.equals(that.pName) : that.pName != null) return false;
+
+            return true;
+        }
+
+        @Override
+        public int hashCode() {
+            int result;
+            result = expectedType.hashCode();
+            result = 31 * result + (pName != null ? pName.hashCode() : 0);
+            result = 31 * result + (useNames ? 1 : 0);
+            result = 31 * result + (binding != null ? binding.hashCode() : 0);
+            result = 31 * result + currentParameter.hashCode();
+            return result;
+        }
+    }
+
+    private void fixGenericParameterTypes(Constructor<T> ctor, Type[] parameterTypes) {
         for (int i = 0; i < parameterTypes.length; i++) {
             Type parameterType = parameterTypes[i];
             if (parameterType instanceof TypeVariable) {
-                parameterTypes[i] = sortedMatchingConstructor.getParameterTypes()[i];
+                parameterTypes[i] = ctor.getParameterTypes()[i];
             }
         }
     }
 
-    public T getComponentInstance(final PicoContainer container, Type into) throws PicoCompositionException {
+    protected class CtorAndAdapters<TYPE> {
+        private final Constructor<TYPE> ctor;
+        private final Parameter[] constructorParameters;
+        private final ComponentAdapter[] injecteeAdapters;
+
+        public CtorAndAdapters(Constructor<TYPE> ctor, Parameter[] parameters, ComponentAdapter[] injecteeAdapters) {
+            this.ctor = ctor;
+            this.constructorParameters = parameters;
+            this.injecteeAdapters = injecteeAdapters;
+        }
+
+        public Constructor<TYPE> getConstructor() {
+            return ctor;
+        }
+
+        public Object[] getParameterArguments(PicoContainer container) {
+            Type[] parameterTypes = ctor.getGenericParameterTypes();
+            // as per fixParameterType()
+            for (int i = 0; i < parameterTypes.length; i++) {
+                Type parameterType = parameterTypes[i];
+                if (parameterType instanceof TypeVariable) {
+                    parameterTypes[i] = ctor.getParameterTypes()[i];
+                }
+            }
+            boxParameters(parameterTypes);            
+            Object[] result = new Object[constructorParameters.length];
+            Annotation[] bindings = getBindings(ctor.getParameterAnnotations());
+            for (int i = 0; i < constructorParameters.length; i++) {
+
+                result[i] = getParameter(container, ctor, i, parameterTypes[i],
+                        bindings[i], constructorParameters[i], injecteeAdapters[i]);
+            }
+            return result;
+        }
+
+        public ComponentAdapter[] getInjecteeAdapters() {
+            return injecteeAdapters;
+        }
+
+        public Parameter[] getParameters() {
+            return constructorParameters;
+        }
+    }
+
+    @Override
+    public T getComponentInstance(final PicoContainer container, @SuppressWarnings("unused") Type into) throws PicoCompositionException {
         if (instantiationGuard == null) {
             instantiationGuard = new ThreadLocalCyclicDependencyGuard<T>() {
+                @Override
+                @SuppressWarnings("synthetic-access")
                 public T run() {
-                    Constructor<T> ctor = null;
-                    try {
-                        if (chosenConstructor == null) {
-                            ctor = getGreediestSatisfiableConstructor(guardedContainer);
-                        }
-                        if (rememberChosenConstructor) {
-                            if (chosenConstructor == null) {
-                                chosenConstructor = ctor;
-                            } else {
-                                ctor = chosenConstructor;
-                            }
-                        }
-                    } catch (AmbiguousComponentResolutionException e) {
-                        e.setComponent(getComponentImplementation());
-                        throw e;
-                    }
+                    CtorAndAdapters<T> ctorAndAdapters = getGreediestSatisfiableConstructor(guardedContainer, getComponentImplementation());
                     ComponentMonitor componentMonitor = currentMonitor();
+                    Constructor<T> ctor = ctorAndAdapters.getConstructor();
                     try {
-                        Object[] parameters = getMemberArguments(guardedContainer, ctor);
+                        Object[] ctorParameters = ctorAndAdapters.getParameterArguments(guardedContainer);
                         ctor = componentMonitor.instantiating(container, ConstructorInjector.this, ctor);
-                        if(ctor == null) {
+                        if(ctorAndAdapters == null) {
                             throw new NullPointerException("Component Monitor " + componentMonitor 
-                                            + " returned a null constructor from method 'instantiating' after passing in " + ctor);
+                                            + " returned a null constructor from method 'instantiating' after passing in " + ctorAndAdapters);
                         }
                         long startTime = System.currentTimeMillis();
-                        T inst = instantiate(ctor, parameters);
-                        componentMonitor.instantiated(container,
-                                                      ConstructorInjector.this,
-                                ctor, inst, parameters, System.currentTimeMillis() - startTime);
+                        T inst = newInstance(ctor, ctorParameters);
+                        componentMonitor.instantiated(container, ConstructorInjector.this,
+                                ctor, inst, ctorParameters, System.currentTimeMillis() - startTime);
                         return inst;
                     } catch (InvocationTargetException e) {
                         componentMonitor.instantiationFailed(container, ConstructorInjector.this, ctor, e);
@@ -229,17 +339,6 @@ public class ConstructorInjector<T> extends SingleMemberInjector<T> {
         return instantiationGuard.observe(getComponentImplementation());
     }
 
-    protected T instantiate(Constructor<T> constructor, Object[] parameters) throws InstantiationException, IllegalAccessException, InvocationTargetException {
-        T inst = newInstance(constructor, parameters);
-        return inst;
-    }
-
-    protected Object[] getMemberArguments(PicoContainer container, final Constructor ctor) {
-        Type[] parameterTypes = ctor.getGenericParameterTypes();
-        fixParameterType(ctor, parameterTypes);
-        return super.getMemberArguments(container, ctor, parameterTypes, getBindings(ctor.getParameterAnnotations()));
-    }
-
     private List<Constructor<T>> getSortedMatchingConstructors() {
         List<Constructor<T>> matchingConstructors = new ArrayList<Constructor<T>>();
         Constructor<T>[] allConstructors = getConstructors();
@@ -272,13 +371,14 @@ public class ConstructorInjector<T> extends SingleMemberInjector<T> {
     public void verify(final PicoContainer container) throws PicoCompositionException {
         if (verifyingGuard == null) {
             verifyingGuard = new ThreadLocalCyclicDependencyGuard() {
+                @Override
                 public Object run() {
-                    final Constructor constructor = getGreediestSatisfiableConstructor(guardedContainer);
+                    final Constructor constructor = getGreediestSatisfiableConstructor(guardedContainer).getConstructor();
                     final Class[] parameterTypes = constructor.getParameterTypes();
                     final Parameter[] currentParameters = parameters != null ? parameters : createDefaultParameters(parameterTypes);
                     for (int i = 0; i < currentParameters.length; i++) {
                         currentParameters[i].verify(container, ConstructorInjector.this, box(parameterTypes[i]),
-                            new ParameterNameBinding(getParanamer(), getComponentImplementation(),  constructor, i),
+                            new ParameterNameBinding(getParanamer(),  constructor, i),
                                 useNames(), getBindings(constructor.getParameterAnnotations())[i]);
                     }
                     return null;
@@ -289,6 +389,7 @@ public class ConstructorInjector<T> extends SingleMemberInjector<T> {
         verifyingGuard.observe(getComponentImplementation());
     }
 
+    @Override
     public String getDescriptor() {
         return "ConstructorInjector-";
     }
