@@ -106,23 +106,26 @@ public class ReflectionLifecycleStrategy extends AbstractMonitoringLifecycleStra
         if (component != null && method != null) {
             try {
                 long str = System.currentTimeMillis();
-                currentMonitor().invoking(null, null, method, component);
+                currentMonitor().invoking(null, null, method, component, new Object[0]);
                 method.invoke(component);
-                currentMonitor().invoked(null, null, method, component, System.currentTimeMillis() - str);
+                currentMonitor().invoked(null, null, method, component, System.currentTimeMillis() - str, new Object[0], null);
             } catch (IllegalAccessException e) {
                 monitorAndThrowReflectionLifecycleException(method, e, component);
             } catch (InvocationTargetException e) {
-                monitorAndThrowReflectionLifecycleException(method, (Exception) e.getCause(), component);
+                monitorAndThrowReflectionLifecycleException(method, e.getCause(), component);
             }
         }
     }
 
     protected void monitorAndThrowReflectionLifecycleException(final Method method,
-                                                             final Exception e,
+                                                             final Throwable e,
                                                              final Object component) {
         RuntimeException re;
         if (e.getCause() instanceof RuntimeException) {
             re = (RuntimeException) e.getCause();
+        // TODO - change lifecycleInvocationFailed to take a throwable in future version
+//        } else if (e.getCause() instanceof Error) {
+//            re = (Error) e.getCause();
         } else {
             re = new RuntimeException("wrapper", e);
         }
